from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import WhatsMinerMake


class M20SV10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M20SV10

    expected_chips = 105
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M20SV20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M20SV20

    expected_chips = 111
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M20SV30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M20SV30

    expected_chips = 140
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256



from pyasic.miners.backends import M3X
from pyasic.miners.device.models import M32V10


class BTMinerM32V10(M3X, M32V10):
    pass


from pyasic.miners.device.models import M32V20


class BTMinerM32V20(M3X, M32V20):
    pass



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import WhatsMinerMake


class M54SPlusPlusVK30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M54SPlusPlusVK30

    expected_chips = 96
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M54SPlusPlusVL30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M54SPlusPlusVL30

    expected_chips = 68
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M54SPlusPlusVL40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M54SPlusPlusVL40

    expected_chips = 90
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import WhatsMinerMake


class M66SVK20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M66SVK20

    expected_chips = 368
    expected_fans = 0
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M66SVK30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M66SVK30

    expected_chips = 384
    expected_fans = 0
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M66SVK40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M66SVK40

    expected_chips = 240
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M66SVK50(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M66SVK50

    expected_chips = 250
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M66SVK60(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M66SVK60

    expected_chips = 250
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M66SVL10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M66SVL10

    expected_chips = 168
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M66SVL20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M66SVL20

    expected_chips = 176
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M66SVL30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M66SVL30

    expected_chips = 192
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M66SVL40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M66SVL40

    expected_chips = 200
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M66SVL50(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M66SVL50

    expected_chips = 210
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256



from __future__ import annotations

from .base import MinerAlgoType
from .hashrate import SHA256HashRate
from .hashrate.unit import SHA256Unit


# make this json serializable
class SHA256Algo(MinerAlgoType):
    hashrate: type[SHA256HashRate] = SHA256HashRate
    unit: type[SHA256Unit] = SHA256Unit

    name = "SHA256"



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import WhatsMinerMake


class M31SV10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M31SV10

    expected_chips = 105
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M31SV20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M31SV20

    expected_chips = 111
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M31SV30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M31SV30

    expected_chips = 117
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M31SV40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M31SV40

    expected_chips = 123
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M31SV50(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M31SV50

    expected_chips = 78
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M31SV60(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M31SV60

    expected_chips = 105
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M31SV70(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M31SV70

    expected_chips = 111
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M31SV80(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M31SV80

    expected_chips = None
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M31SV90(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M31SV90

    expected_chips = 117
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M31SVE10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M31SVE10

    expected_chips = 70
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M31SVE20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M31SVE20

    expected_chips = 74
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M31SVE30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M31SVE30

    expected_chips = None
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256



from __future__ import annotations

from dataclasses import field
from typing import TypeVar, Union

from pyasic.config.base import MinerConfigOption, MinerConfigValue


class StandardTuneAlgo(MinerConfigValue):
    mode: str = field(init=False, default="standard")

    def as_epic(self) -> str:
        return VOptAlgo().as_epic()


class VOptAlgo(MinerConfigValue):
    mode: str = field(init=False, default="voltage_optimizer")

    def as_epic(self) -> str:
        return "VoltageOptimizer"


class BoardTuneAlgo(MinerConfigValue):
    mode: str = field(init=False, default="board_tune")

    def as_epic(self) -> str:
        return "BoardTune"


class ChipTuneAlgo(MinerConfigValue):
    mode: str = field(init=False, default="chip_tune")

    def as_epic(self) -> str:
        return "ChipTune"


class TunerAlgo(MinerConfigOption):
    standard = StandardTuneAlgo
    voltage_optimizer = VOptAlgo
    board_tune = BoardTuneAlgo
    chip_tune = ChipTuneAlgo

    @classmethod
    def default(cls) -> TunerAlgoType:
        return cls.standard()

    @classmethod
    def from_dict(cls, dict_conf: dict | None) -> TunerAlgoType:
        mode = dict_conf.get("mode")
        if mode is None:
            return cls.default()

        cls_attr = getattr(cls, mode)
        if cls_attr is not None:
            return cls_attr().from_dict(dict_conf)


TunerAlgoType = TypeVar(
    "TunerAlgoType",
    bound=Union[
        StandardTuneAlgo,
        VOptAlgo,
        BoardTuneAlgo,
        ChipTuneAlgo,
    ],
)



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import WhatsMinerMake


class M53HVH10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M53HVH10

    expected_chips = 56
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256



from .base import AlgoHashRateType
from .blake256 import Blake256HashRate
from .eaglesong import EaglesongHashRate
from .equihash import EquihashHashRate
from .ethash import EtHashHashRate
from .handshake import HandshakeHashRate
from .kadena import KadenaHashRate
from .kheavyhash import KHeavyHashHashRate
from .scrypt import ScryptHashRate
from .sha256 import SHA256HashRate
from .x11 import X11HashRate


class AlgoHashRate:
    SHA256 = SHA256HashRate
    SCRYPT = ScryptHashRate
    KHEAVYHASH = KHeavyHashHashRate
    KADENA = KadenaHashRate
    HANDSHAKE = HandshakeHashRate
    X11 = X11HashRate
    BLAKE256 = Blake256HashRate
    EAGLESONG = EaglesongHashRate
    ETHASH = EtHashHashRate
    EQUIHASH = EquihashHashRate



from pyasic.miners.backends import M2X
from pyasic.miners.device.models import M20PV10


class BTMinerM20PV10(M2X, M20PV10):
    pass


from pyasic.miners.device.models import M20PV30


class BTMinerM20PV30(M2X, M20PV30):
    pass



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import WhatsMinerMake


class M56SPlusVJ30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M56SPlusVJ30

    expected_chips = 176
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M56SPlusVK30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M56SPlusVK30

    expected_chips = 108
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M56SPlusVK40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M56SPlusVK40

    expected_chips = 114
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M56SPlusVK50(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M56SPlusVK50

    expected_chips = 120
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import WhatsMinerMake


class M30SPlusPlusV10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusV10

    expected_chips = 255
    expected_fans = 2
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M30SPlusPlusV20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusV20

    expected_chips = 255
    expected_fans = 2
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M30SPlusPlusVE30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVE30

    expected_chips = 215
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVE40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVE40

    expected_chips = 225
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVE50(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVE50

    expected_chips = 235
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVF40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVF40

    expected_chips = 156
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVG30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVG30

    expected_chips = 111
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVG40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVG40

    expected_chips = 117
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVG50(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVG50

    expected_chips = 123
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVH10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVH10

    expected_chips = 82
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVH100(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVH100

    expected_chips = 82
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVH110(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVH110

    expected_chips = 105
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVH20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVH20

    expected_chips = 86
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVH30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVH30

    expected_chips = 111
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVH40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVH40

    expected_chips = 70
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVH50(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVH50

    expected_chips = 74
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVH60(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVH60

    expected_chips = 78
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVH70(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVH70

    expected_chips = 70
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVH80(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVH80

    expected_chips = 74
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVH90(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVH90

    expected_chips = 78
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVI30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVI30

    expected_chips = 111
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVJ20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVJ20

    expected_chips = 70
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVJ30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVJ30

    expected_chips = 74
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVJ50(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVJ50

    expected_chips = 82
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVJ60(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVJ60

    expected_chips = 111
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVJ70(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVJ70

    expected_chips = 117
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SPlusPlusVK30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVK30

    expected_chips = 74
    expected_fans = 2
    expected_hashboards = 2
    algo = MinerAlgo.SHA256


class M30SPlusPlusVK40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SPlusPlusVK40

    expected_chips = 105
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256



from __future__ import annotations

from .base import AlgoHashRateUnitType


class EquihashUnit(AlgoHashRateUnitType):
    H = 1
    KH = int(H) * 1000
    MH = int(KH) * 1000
    GH = int(MH) * 1000
    TH = int(GH) * 1000
    PH = int(TH) * 1000
    EH = int(PH) * 1000
    ZH = int(EH) * 1000

    default = KH

    def __str__(self):
        if self.value == self.H:
            return "Sol/s"
        if self.value == self.KH:
            return "KSol/s"
        if self.value == self.MH:
            return "MSol/s"
        if self.value == self.GH:
            return "GSol/s"
        if self.value == self.TH:
            return "TSol/s"
        if self.value == self.PH:
            return "PSol/s"
        if self.value == self.EH:
            return "ESol/s"
        if self.value == self.ZH:
            return "ZSol/s"



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import WhatsMinerMake


class M33SPlusVG20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M33SPlusVG20

    expected_chips = 112
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M33SPlusVG30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M33SPlusVG30

    expected_chips = 162
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M33SPlusVH20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M33SPlusVH20

    expected_chips = 100
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M33SPlusVH30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M33SPlusVH30

    expected_chips = None
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import WhatsMinerMake


class M60SVK10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M60SVK10

    expected_chips = 215
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M60SVK20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M60SVK20

    expected_chips = 235
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M60SVK30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M60SVK30

    expected_chips = 245
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M60SVK40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M60SVK40

    expected_chips = 225
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M60SVL10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M60SVL10

    expected_chips = 147
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M60SVL20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M60SVL20

    expected_chips = 164
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M60SVL30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M60SVL30

    expected_chips = 172
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M60SVL40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M60SVL40

    expected_chips = 180
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M60SVL50(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M60SVL50

    expected_chips = 188
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M60SVL60(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M60SVL60

    expected_chips = 196
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M60SVL70(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M60SVL70

    expected_chips = 141
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256



# ------------------------------------------------------------------------------
#  Copyright 2022 Upstream Data Inc                                            -
#                                                                              -
#  Licensed under the Apache License, Version 2.0 (the "License");             -
#  you may not use this file except in compliance with the License.            -
#  You may obtain a copy of the License at                                     -
#                                                                              -
#      http://www.apache.org/licenses/LICENSE-2.0                              -
#                                                                              -
#  Unless required by applicable law or agreed to in writing, software         -
#  distributed under the License is distributed on an "AS IS" BASIS,           -
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.    -
#  See the License for the specific language governing permissions and         -
#  limitations under the License.                                              -
# ------------------------------------------------------------------------------
from __future__ import annotations

import asyncio
import hashlib
import json
from typing import Any

import httpx

from pyasic import settings
from pyasic.web.base import BaseWebAPI


class AvalonMinerWebAPI(BaseWebAPI):
    def __init__(self, ip: str) -> None:
        """Initialize the modern Avalonminer API client with a specific IP address.

        Args:
            ip (str): IP address of the Avalonminer device.
        """
        super().__init__(ip)
        self.username = "root"
        self.pwd = settings.get("default_avalonminer_web_password", "root")

    async def send_command(
        self,
        command: str | bytes,
        ignore_errors: bool = False,
        allow_warning: bool = True,
        **parameters: Any,
    ) -> dict:
        """Send a command to the Avalonminer device using HTTP digest authentication.

        Args:
            command (str | bytes): The CGI command to send.
            ignore_errors (bool): If True, ignore any HTTP errors.
            allow_warning (bool): If True, proceed with warnings.
            **parameters: Arbitrary keyword arguments to be sent as parameters in the request.

        Returns:
            dict: The JSON response from the device or an empty dictionary if an error occurs.
        """
        cookie_data = "ff0000ff" + hashlib.sha256(self.pwd.encode()).hexdigest()[:24]

        url = f"http://{self.ip}:{self.port}/{command}.cgi"
        try:
            async with httpx.AsyncClient(transport=settings.transport()) as client:
                client.cookies.set("auth", cookie_data)
                resp = await client.get(url)
                raw_data = resp.text.replace("minerinfoCallback(", "").replace(");", "")
                return json.loads(raw_data)
        except httpx.HTTPError:
            pass
        return {}

    async def multicommand(
        self, *commands: str, ignore_errors: bool = False, allow_warning: bool = True
    ) -> dict:
        async with httpx.AsyncClient(transport=settings.transport()) as client:
            cookie_data = (
                "ff0000ff" + hashlib.sha256(self.pwd.encode()).hexdigest()[:24]
            )
            client.cookies.set("auth", cookie_data)
            tasks = [
                asyncio.create_task(self._handle_multicommand(client, command))
                for command in commands
            ]
            all_data = await asyncio.gather(*tasks)

        data = {}
        for item in all_data:
            data.update(item)

        data["multicommand"] = True
        return data

    async def _handle_multicommand(
        self, client: httpx.AsyncClient, command: str
    ) -> dict:
        try:
            url = f"http://{self.ip}:{self.port}/{command}.cgi"
            resp = await client.get(url)
            raw_data = resp.text.replace("minerinfoCallback(", "").replace(");", "")
            return json.loads(raw_data)
        except httpx.HTTPError:
            pass
        return {}

    async def minerinfo(self):
        return await self.send_command("get_minerinfo")

    async def home(self):
        return await self.send_command("get_home")



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import AvalonMinerMake


class AvalonNano3(AvalonMinerMake):
    raw_model = MinerModel.AVALONMINER.AvalonNano3

    expected_hashboards = 1
    expected_chips = 10
    expected_fans = 1
    algo = MinerAlgo.SHA256



from __future__ import annotations

from .base import MinerAlgoType
from .hashrate import ScryptHashRate
from .hashrate.unit import ScryptUnit


class ScryptAlgo(MinerAlgoType):
    hashrate: type[ScryptHashRate] = ScryptHashRate
    unit: type[ScryptUnit] = ScryptUnit

    name = "Scrypt"



from __future__ import annotations

from .base import MinerAlgoType
from .hashrate import EquihashHashRate
from .hashrate.unit import EquihashUnit


# make this json serializable
class EquihashAlgo(MinerAlgoType):
    hashrate: type[EquihashHashRate] = EquihashHashRate
    unit: type[EquihashUnit] = EquihashUnit

    name = "Equihash"



from pyasic.miners.backends import M6X
from pyasic.miners.device.models import M63VK10


class BTMinerM63VK10(M6X, M63VK10):
    pass


from pyasic.miners.device.models import M63VK20


class BTMinerM63VK20(M6X, M63VK20):
    pass


from pyasic.miners.device.models import M63VK30


class BTMinerM63VK30(M6X, M63VK30):
    pass


from pyasic.miners.device.models import M63VL10


class BTMinerM63VL10(M6X, M63VL10):
    pass


from pyasic.miners.device.models import M63VL30


class BTMinerM63VL30(M6X, M63VL30):
    pass



from .base import MinerAlgoType
from .blake256 import Blake256Algo
from .eaglesong import EaglesongAlgo
from .equihash import EquihashAlgo
from .ethash import EtHashAlgo
from .handshake import HandshakeAlgo
from .hashrate import *
from .hashrate.unit import *
from .kadena import KadenaAlgo
from .kheavyhash import KHeavyHashAlgo
from .scrypt import ScryptAlgo
from .sha256 import SHA256Algo
from .x11 import X11Algo


class MinerAlgo:
    SHA256 = SHA256Algo
    SCRYPT = ScryptAlgo
    KHEAVYHASH = KHeavyHashAlgo
    KADENA = KadenaAlgo
    HANDSHAKE = HandshakeAlgo
    X11 = X11Algo
    BLAKE256 = Blake256Algo
    EAGLESONG = EaglesongAlgo
    ETHASH = EtHashAlgo
    EQUIHASH = EquihashAlgo



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import WhatsMinerMake


class M50VE30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50VE30

    expected_chips = 255
    expected_fans = 2
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M50VG30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50VG30

    expected_chips = 156
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50VH10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50VH10

    expected_chips = 86
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50VH20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50VH20

    expected_chips = 111
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50VH30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50VH30

    expected_chips = 117
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50VH40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50VH40

    expected_chips = 84
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50VH50(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50VH50

    expected_chips = 105
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50VH60(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50VH60

    expected_chips = 84
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50VH70(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50VH70

    expected_chips = 105
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50VH80(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50VH80

    expected_chips = 111
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50VH90(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50VH90

    expected_chips = 117
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50VJ10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50VJ10

    expected_chips = 86
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50VJ20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50VJ20

    expected_chips = 111
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50VJ30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50VJ30

    expected_chips = 117
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50VJ40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50VJ40

    expected_chips = 123
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50VJ60(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50VJ60

    expected_chips = 164
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50VK40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50VK40

    expected_chips = 111
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50VK50(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50VK50

    expected_chips = 117
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256



from pyasic.miners.backends import M6X
from pyasic.miners.device.models import M61VK10


class BTMinerM61VK10(M6X, M61VK10):
    pass


from pyasic.miners.device.models import M61VK20


class BTMinerM61VK20(M6X, M61VK20):
    pass


from pyasic.miners.device.models import M61VK30


class BTMinerM61VK30(M6X, M61VK30):
    pass


from pyasic.miners.device.models import M61VK40


class BTMinerM61VK40(M6X, M61VK40):
    pass


from pyasic.miners.device.models import M61VL10


class BTMinerM61VL10(M6X, M61VL10):
    pass


from pyasic.miners.device.models import M61VL30


class BTMinerM61VL30(M6X, M61VL30):
    pass


from pyasic.miners.device.models import M61VL40


class BTMinerM61VL40(M6X, M61VL40):
    pass


from pyasic.miners.device.models import M61VL50


class BTMinerM61VL50(M6X, M61VL50):
    pass


from pyasic.miners.device.models import M61VL60


class BTMinerM61VL60(M6X, M61VL60):
    pass



from pyasic.config.base import MinerConfigValue


class MiningPreset(MinerConfigValue):
    name: str | None = None
    power: int | None = None
    hashrate: int | None = None
    tuned: bool | None = None
    modded_psu: bool | None = None
    frequency: int | None = None
    voltage: float | None = None

    def as_vnish(self) -> dict:
        if self.name is not None:
            return {"preset": self.name}
        return {}

    @classmethod
    def from_vnish(cls, web_preset: dict):
        name = web_preset["name"]
        hr_power_split = web_preset["pretty"].split("~")
        if len(hr_power_split) == 1:
            power = None
            hashrate = None
        else:
            power = hr_power_split[0].replace("watt", "").strip()
            hashrate = (
                hr_power_split[1]
                .replace("TH", "")
                .replace("GH", "")
                .replace("MH", "")
                .replace(" LC", "")
                .strip()
            )
        tuned = web_preset["status"] == "tuned"
        modded_psu = web_preset["modded_psu_required"]
        return cls(
            name=name,
            power=power,
            hashrate=hashrate,
            tuned=tuned,
            modded_psu=modded_psu,
        )

    @classmethod
    def from_luxos(cls, profile: dict):
        return cls(
            name=profile["Profile Name"],
            power=profile["Watts"],
            hashrate=round(profile["Hashrate"]),
            tuned=profile["IsTuned"],
            frequency=profile["Frequency"],
            voltage=profile["Voltage"],
        )



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import WhatsMinerMake


class M50SVH10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SVH10

    expected_chips = None
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SVH20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SVH20

    expected_chips = 135
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SVH30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SVH30

    expected_chips = 156
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SVH40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SVH40

    expected_chips = 148
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SVH50(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SVH50

    expected_chips = 135
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SVJ10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SVJ10

    expected_chips = 111
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SVJ20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SVJ20

    expected_chips = 117
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SVJ30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SVJ30

    expected_chips = 123
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SVJ40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SVJ40

    expected_chips = 129
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SVJ50(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SVJ50

    expected_chips = 135
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SVK10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SVK10

    expected_chips = 78
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SVK20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SVK20

    expected_chips = 111
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SVK30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SVK30

    expected_chips = 117
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SVK50(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SVK50

    expected_chips = 105
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SVK60(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SVK60

    expected_chips = 111
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SVK70(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SVK70

    expected_chips = 123
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SVK80(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SVK80

    expected_chips = 86
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SVL20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SVL20

    expected_chips = 78
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SVL30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SVL30

    expected_chips = 82
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import AuradineMake


class AuradineAT2860(AuradineMake):
    raw_model = MinerModel.AURADINE.AT2860

    expected_fans = 4
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class AuradineAT2880(AuradineMake):
    raw_model = MinerModel.AURADINE.AT2880

    expected_fans = 4
    expected_hashboards = 3
    algo = MinerAlgo.SHA256



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import WhatsMinerMake


class M20PV10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M20PV10

    expected_chips = 156
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M20PV30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M20PV30

    expected_chips = 148
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import WhatsMinerMake


class M61SVL10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M61SVL10

    expected_chips = 164
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M61SVL20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M61SVL20

    expected_chips = 172
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M61SVL30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M61SVL30

    expected_chips = 180
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import WhatsMinerMake


class M66SPlusVK30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M66SPlusVK30

    expected_chips = 440
    expected_fans = 0
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M66SPlusVL10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M66SPlusVL10

    expected_chips = 220
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M66SPlusVL20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M66SPlusVL20

    expected_chips = 230
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M66SPlusVL30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M66SPlusVL30

    expected_chips = 240
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M66SPlusVL40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M66SPlusVL40

    expected_chips = 250
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M66SPlusVL60(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M66SPlusVL60

    expected_chips = 200
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256



from pyasic.miners.backends import M3X
from pyasic.miners.device.models import M31V10


class BTMinerM31V10(M3X, M31V10):
    pass


from pyasic.miners.device.models import M31V20


class BTMinerM31V20(M3X, M31V20):
    pass



# ------------------------------------------------------------------------------
#  Copyright 2024 Upstream Data Inc                                            -
#                                                                              -
#  Licensed under the Apache License, Version 2.0 (the "License");             -
#  you may not use this file except in compliance with the License.            -
#  You may obtain a copy of the License at                                     -
#                                                                              -
#      http://www.apache.org/licenses/LICENSE-2.0                              -
#                                                                              -
#  Unless required by applicable law or agreed to in writing, software         -
#  distributed under the License is distributed on an "AS IS" BASIS,           -
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.    -
#  See the License for the specific language governing permissions and         -
#  limitations under the License.                                              -
# ------------------------------------------------------------------------------

from pyasic.rpc.bmminer import BMMinerRPCAPI


class CCMinerRPCAPI(BMMinerRPCAPI):
    """An abstraction of the CCMiner API.

    Each method corresponds to an API command in CCMiner.

    This class abstracts use of the CCMiner API, as well as the
    methods for sending commands to it.  The `self.send_command()`
    function handles sending a command to the miner asynchronously, and
    as such is the base for many of the functions in this class, which
    rely on it to send the command for them.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.port = 8359



from pyasic.device.algorithm.hashrate.unit.scrypt import ScryptUnit
from pyasic.device.algorithm.scrypt import ScryptHashRate
from pyasic.miners.backends import BlackMiner
from pyasic.miners.device.models import D1


class VolcMinerD1(BlackMiner, D1):
    sticker_hashrate = ScryptHashRate(rate=15.15, unit=ScryptUnit.GH)



from pyasic.miners.backends import M6X
from pyasic.miners.device.models import M66SPlusVK30


class BTMinerM66SPlusVK30(M6X, M66SPlusVK30):
    pass


from pyasic.miners.device.models import M66SPlusVL10


class BTMinerM66SPlusVL10(M6X, M66SPlusVL10):
    pass


from pyasic.miners.device.models import M66SPlusVL20


class BTMinerM66SPlusVL20(M6X, M66SPlusVL20):
    pass


from pyasic.miners.device.models import M66SPlusVL30


class BTMinerM66SPlusVL30(M6X, M66SPlusVL30):
    pass


from pyasic.miners.device.models import M66SPlusVL40


class BTMinerM66SPlusVL40(M6X, M66SPlusVL40):
    pass


from pyasic.miners.device.models import M66SPlusVL60


class BTMinerM66SPlusVL60(M6X, M66SPlusVL60):
    pass



from __future__ import annotations

from .base import MinerAlgoType
from .hashrate import HandshakeHashRate
from .hashrate.unit import HandshakeUnit


# make this json serializable
class HandshakeAlgo(MinerAlgoType):
    hashrate: type[HandshakeHashRate] = HandshakeHashRate
    unit: type[HandshakeUnit] = HandshakeUnit

    name = "Handshake"



from pyasic.miners.backends import M5X
from pyasic.miners.device.models import M53VH30


class BTMinerM53VH30(M5X, M53VH30):
    pass


from pyasic.miners.device.models import M53VH40


class BTMinerM53VH40(M5X, M53VH40):
    pass


from pyasic.miners.device.models import M53VH50


class BTMinerM53VH50(M5X, M53VH50):
    pass


from pyasic.miners.device.models import M53VK30


class BTMinerM53VK30(M5X, M53VK30):
    pass


from pyasic.miners.device.models import M53VK60


class BTMinerM53VK60(M5X, M53VK60):
    pass



from enum import Enum
from typing import Optional
from urllib.parse import urlparse

from pydantic import BaseModel, computed_field, model_serializer
from typing_extensions import Self


class Scheme(Enum):
    STRATUM_V1 = "stratum+tcp"
    STRATUM_V2 = "stratum2+tcp"
    STRATUM_V1_SSL = "stratum+ssl"


class PoolUrl(BaseModel):
    scheme: Scheme
    host: str
    port: Optional[int] = None
    pubkey: Optional[str] = None

    @model_serializer
    def serialize(self):
        return str(self)

    def __str__(self) -> str:
        port_str = f":{self.port}" if self.port is not None else ""
        if self.scheme == Scheme.STRATUM_V2 and self.pubkey:
            return f"{self.scheme.value}://{self.host}{port_str}/{self.pubkey}"
        else:
            return f"{self.scheme.value}://{self.host}{port_str}"

    @classmethod
    def from_str(cls, url: str) -> Self | None:
        parsed_url = urlparse(url)
        if not parsed_url.hostname:
            return None
        # Если схема отсутствует, используем схему по умолчанию
        scheme = Scheme(parsed_url.scheme) if parsed_url.scheme.strip() != "" else Scheme.STRATUM_V1
        host = parsed_url.hostname
        port = parsed_url.port  # может быть None, если порт не указан
        pubkey = parsed_url.path.lstrip("/") if scheme == Scheme.STRATUM_V2 else None
        return cls(scheme=scheme, host=host, port=port, pubkey=pubkey)


class PoolMetrics(BaseModel):
    """
    A dataclass to standardize pool metrics returned from miners.
    Attributes:
        accepted: Number of accepted shares.
        rejected: Number of rejected shares.
        get_failures: Number of failures in obtaining work from the pool.
        remote_failures: Number of failures communicating with the pool server.
        active: Indicates if the miner is connected to the stratum server.
        alive: Indicates if a pool is alive.
        url: URL of the pool.
        index: Index of the pool.
        user: Username for the pool.
        pool_rejected_percent: Percentage of rejected shares by the pool.
        pool_stale_percent: Percentage of stale shares by the pool.
    """

    url: Optional[PoolUrl]
    accepted: Optional[int] = None
    rejected: Optional[int] = None
    get_failures: Optional[int] = None
    remote_failures: Optional[int] = None
    active: Optional[bool] = None
    alive: Optional[bool] = None
    index: Optional[int] = None
    user: Optional[str] = None

    @computed_field  # type: ignore[misc]
    @property
    def pool_rejected_percent(self) -> float:
        """Calculate and return the percentage of rejected shares."""
        total = (self.accepted or 0) + (self.rejected or 0)
        return self._calculate_percentage(self.rejected, total)

    @computed_field  # type: ignore[misc]
    @property
    def pool_stale_percent(self) -> float:
        """Calculate and return the percentage of stale shares."""
        total = (self.accepted or 0) + (self.rejected or 0)
        return self._calculate_percentage(self.get_failures, total)

    @staticmethod
    def _calculate_percentage(value: Optional[int], total: int) -> float:
        """Calculate the percentage."""
        if value is None or total == 0:
            return 0
        return (value / total) * 100



from pyasic.miners.backends import M5X
from pyasic.miners.device.models import M50VE30


class BTMinerM50VE30(M5X, M50VE30):
    pass


from pyasic.miners.device.models import M50VG30


class BTMinerM50VG30(M5X, M50VG30):
    pass


from pyasic.miners.device.models import M50VH10


class BTMinerM50VH10(M5X, M50VH10):
    pass


from pyasic.miners.device.models import M50VH20


class BTMinerM50VH20(M5X, M50VH20):
    pass


from pyasic.miners.device.models import M50VH30


class BTMinerM50VH30(M5X, M50VH30):
    pass


from pyasic.miners.device.models import M50VH40


class BTMinerM50VH40(M5X, M50VH40):
    pass


from pyasic.miners.device.models import M50VH50


class BTMinerM50VH50(M5X, M50VH50):
    pass


from pyasic.miners.device.models import M50VH60


class BTMinerM50VH60(M5X, M50VH60):
    pass


from pyasic.miners.device.models import M50VH70


class BTMinerM50VH70(M5X, M50VH70):
    pass


from pyasic.miners.device.models import M50VH80


class BTMinerM50VH80(M5X, M50VH80):
    pass


from pyasic.miners.device.models import M50VH90


class BTMinerM50VH90(M5X, M50VH90):
    pass


from pyasic.miners.device.models import M50VJ10


class BTMinerM50VJ10(M5X, M50VJ10):
    pass


from pyasic.miners.device.models import M50VJ20


class BTMinerM50VJ20(M5X, M50VJ20):
    pass


from pyasic.miners.device.models import M50VJ30


class BTMinerM50VJ30(M5X, M50VJ30):
    pass


from pyasic.miners.device.models import M50VJ40


class BTMinerM50VJ40(M5X, M50VJ40):
    pass


from pyasic.miners.device.models import M50VJ60


class BTMinerM50VJ60(M5X, M50VJ60):
    pass


from pyasic.miners.device.models import M50VK40


class BTMinerM50VK40(M5X, M50VK40):
    pass


from pyasic.miners.device.models import M50VK50


class BTMinerM50VK50(M5X, M50VK50):
    pass



from __future__ import annotations

from .base import MinerAlgoType
from .hashrate import EaglesongHashRate
from .hashrate.unit import EaglesongUnit


# make this json serializable
class EaglesongAlgo(MinerAlgoType):
    hashrate: type[EaglesongHashRate] = EaglesongHashRate
    unit: type[EaglesongUnit] = EaglesongUnit

    name = "Eaglesong"



# ------------------------------------------------------------------------------
#  Copyright 2022 Upstream Data Inc                                            -
#                                                                              -
#  Licensed under the Apache License, Version 2.0 (the "License");             -
#  you may not use this file except in compliance with the License.            -
#  You may obtain a copy of the License at                                     -
#                                                                              -
#      http://www.apache.org/licenses/LICENSE-2.0                              -
#                                                                              -
#  Unless required by applicable law or agreed to in writing, software         -
#  distributed under the License is distributed on an "AS IS" BASIS,           -
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.    -
#  See the License for the specific language governing permissions and         -
#  limitations under the License.                                              -
# ------------------------------------------------------------------------------
from __future__ import annotations

import json
from pathlib import Path
from typing import Any

import aiofiles
import httpx

from pyasic import settings
from pyasic.web.base import BaseWebAPI


class HiveonWebAPI(BaseWebAPI):
    def __init__(self, ip: str) -> None:
        """Initialize the old Antminer API client with a specific IP address.

        Args:
            ip (str): IP address of the Antminer device.
        """
        super().__init__(ip)
        self.username = "root"
        self.pwd = settings.get("default_hive_web_password", "root")

    async def send_command(
        self,
        command: str | bytes,
        ignore_errors: bool = False,
        allow_warning: bool = True,
        privileged: bool = False,
        **parameters: Any,
    ) -> dict:
        """Send a command to the Antminer device using HTTP digest authentication.

        Args:
            command (str | bytes): The CGI command to send.
            ignore_errors (bool): If True, ignore any HTTP errors.
            allow_warning (bool): If True, proceed with warnings.
            privileged (bool): If set to True, requires elevated privileges.
            **parameters: Arbitrary keyword arguments to be sent as parameters in the request.

        Returns:
            dict: The JSON response from the device or an empty dictionary if an error occurs.
        """
        url = f"http://{self.ip}:{self.port}/cgi-bin/{command}.cgi"
        auth = httpx.DigestAuth(self.username, self.pwd)
        try:
            async with httpx.AsyncClient(transport=settings.transport()) as client:
                if parameters:
                    data = await client.post(
                        url,
                        data=parameters,
                        auth=auth,
                        timeout=settings.get("api_function_timeout", 3),
                    )
                else:
                    data = await client.get(url, auth=auth)
        except httpx.HTTPError:
            pass
        else:
            if data.status_code == 200:
                try:
                    return data.json()
                except json.decoder.JSONDecodeError:
                    pass

    async def multicommand(
        self, *commands: str, ignore_errors: bool = False, allow_warning: bool = True
    ) -> dict:
        """Execute multiple commands simultaneously.

        Args:
            *commands (str): Multiple command strings to be executed.
            ignore_errors (bool): If True, ignore any HTTP errors.
            allow_warning (bool): If True, proceed with warnings.

        Returns:
            dict: A dictionary containing the results of all commands executed.
        """
        data = {k: None for k in commands}
        auth = httpx.DigestAuth(self.username, self.pwd)
        async with httpx.AsyncClient(transport=settings.transport()) as client:
            for command in commands:
                try:
                    url = f"http://{self.ip}/cgi-bin/{command}.cgi"
                    ret = await client.get(url, auth=auth)
                except httpx.HTTPError:
                    pass
                else:
                    if ret.status_code == 200:
                        try:
                            json_data = ret.json()
                            data[command] = json_data
                        except json.decoder.JSONDecodeError:
                            pass
        return data

    async def get_system_info(self) -> dict:
        """Retrieve system information from the miner.

        Returns:
            dict: A dictionary containing system information of the miner.
        """
        return await self.send_command("get_system_info")

    async def get_network_info(self) -> dict:
        """Retrieve system information from the miner.

        Returns:
            dict: A dictionary containing system information of the miner.
        """
        return await self.send_command("get_network_info")

    async def blink(self, blink: bool) -> dict:
        """Control the blinking of the LED on the miner device.

        Args:
            blink (bool): True to start blinking, False to stop.

        Returns:
            dict: A dictionary response from the device after the command execution.
        """
        if blink:
            return await self.send_command("blink", action="startBlink")
        return await self.send_command("blink", action="stopBlink")

    async def reboot(self) -> dict:
        """Reboot the miner device.

        Returns:
            dict: A dictionary response from the device confirming the reboot command.
        """
        return await self.send_command("reboot")

    async def get_blink_status(self) -> dict:
        """Check the status of the LED blinking on the miner.

        Returns:
            dict: A dictionary indicating whether the LED is currently blinking.
        """
        return await self.send_command("blink", action="onPageLoaded")

    async def get_miner_conf(self) -> dict:
        """Retrieve the miner configuration from the Antminer device.

        Returns:
            dict: A dictionary containing the current configuration of the miner.
        """
        return await self.send_command("get_miner_conf")

    async def set_miner_conf(self, conf: dict) -> dict:
        """Set the configuration for the miner.

        Args:
            conf (dict): A dictionary of configuration settings to apply to the miner.

        Returns:
            dict: A dictionary response from the device after setting the configuration.
        """
        return await self.send_command("set_miner_conf", **conf)

    async def stats(self) -> dict:
        """Retrieve detailed statistical data of the mining operation.

        Returns:
            dict: Detailed statistics of the miner's operation.
        """
        return await self.send_command("miner_stats")

    async def summary(self) -> dict:
        """Get a summary of the miner's status and performance.

        Returns:
            dict: A summary of the miner's current operational status.
        """
        return await self.send_command("miner_summary")

    async def pools(self) -> dict:
        """Retrieve current pool information associated with the miner.

        Returns:
            dict: Information about the mining pools configured in the miner.
        """
        return await self.send_command("miner_pools")

    async def update_firmware(self, file: Path, keep_settings: bool = True) -> dict:
        """Perform a system update by uploading a firmware file and sending a command to initiate the update."""

        async with aiofiles.open(file, "rb") as firmware:
            file_content = await firmware.read()

        parameters = {
            "file": (file.name, file_content, "application/octet-stream"),
            "filename": file.name,
            "keep_settings": keep_settings,
        }

        return await self.send_command(command="upgrade", **parameters)



from pyasic.rpc.base import BaseMinerRPCAPI


class MaraRPCAPI(BaseMinerRPCAPI):
    """An abstraction of the MaraFW API.

    Each method corresponds to an API command in MaraFW.

    No documentation for this API is currently publicly available.

    Additionally, every command not included here just returns the result of the `summary` command.

    This class abstracts use of the MaraFW API, as well as the
    methods for sending commands to it.  The `self.send_command()`
    function handles sending a command to the miner asynchronously, and
    as such is the base for many of the functions in this class, which
    rely on it to send the command for them.
    """

    async def summary(self):
        return await self.send_command("summary")

    async def devs(self):
        return await self.send_command("devs")

    async def pools(self):
        return await self.send_command("pools")

    async def stats(self):
        return await self.send_command("stats")

    async def version(self):
        return await self.send_command("version")



# ------------------------------------------------------------------------------
#  Copyright 2022 Upstream Data Inc                                            -
#                                                                              -
#  Licensed under the Apache License, Version 2.0 (the "License");             -
#  you may not use this file except in compliance with the License.            -
#  You may obtain a copy of the License at                                     -
#                                                                              -
#      http://www.apache.org/licenses/LICENSE-2.0                              -
#                                                                              -
#  Unless required by applicable law or agreed to in writing, software         -
#  distributed under the License is distributed on an "AS IS" BASIS,           -
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.    -
#  See the License for the specific language governing permissions and         -
#  limitations under the License.                                              -
# ------------------------------------------------------------------------------
from typing import List, Optional

from pyasic import APIError, MinerConfig
from pyasic.data import Fan, HashBoard, X19Error
from pyasic.data.error_codes import MinerErrorData
from pyasic.data.pools import PoolMetrics, PoolUrl
from pyasic.device.algorithm import AlgoHashRate
from pyasic.miners.data import (
    DataFunction,
    DataLocations,
    DataOptions,
    WebAPICommand,
)
from pyasic.miners.device.firmware import StockFirmware
from pyasic.web.elphapex import ElphapexWebAPI

ELPHAPEX_DATA_LOC = DataLocations(
    **{
        str(DataOptions.MAC): DataFunction(
            "_get_mac",
            [WebAPICommand("web_get_system_info", "get_system_info")],
        ),
        str(DataOptions.API_VERSION): DataFunction(
            "_get_api_ver",
            [WebAPICommand("web_summary", "summary")],
        ),
        str(DataOptions.FW_VERSION): DataFunction(
            "_get_fw_ver",
            [WebAPICommand("web_get_system_info", "get_system_info")],
        ),
        str(DataOptions.HOSTNAME): DataFunction(
            "_get_hostname",
            [WebAPICommand("web_get_system_info", "get_system_info")],
        ),
        str(DataOptions.HASHBOARDS): DataFunction(
            "_get_hashboards",
            [WebAPICommand("web_stats", "stats")],
        ),
        str(DataOptions.EXPECTED_HASHRATE): DataFunction(
            "_get_expected_hashrate",
            [WebAPICommand("web_stats", "stats")],
        ),
        str(DataOptions.FANS): DataFunction(
            "_get_fans",
            [WebAPICommand("web_stats", "stats")],
        ),
        str(DataOptions.ERRORS): DataFunction(
            "_get_errors",
            [WebAPICommand("web_summary", "summary")],
        ),
        str(DataOptions.FAULT_LIGHT): DataFunction(
            "_get_fault_light",
            [WebAPICommand("web_get_blink_status", "get_blink_status")],
        ),
        str(DataOptions.IS_MINING): DataFunction(
            "_is_mining",
            [WebAPICommand("web_get_miner_conf", "get_miner_conf")],
        ),
        str(DataOptions.UPTIME): DataFunction(
            "_get_uptime",
            [WebAPICommand("web_summary", "summary")],
        ),
        str(DataOptions.POOLS): DataFunction(
            "_get_pools",
            [WebAPICommand("web_pools", "pools")],
        ),
    }
)


class ElphapexMiner(StockFirmware):
    """Handler for Elphapex miners."""

    _web_cls = ElphapexWebAPI
    web: ElphapexWebAPI

    data_locations = ELPHAPEX_DATA_LOC

    async def get_config(self) -> MinerConfig:
        data = await self.web.get_miner_conf()
        if data:
            self.config = MinerConfig.from_elphapex(data)
        return self.config

    async def send_config(self, config: MinerConfig, user_suffix: str = None) -> None:
        self.config = config
        await self.web.set_miner_conf(config.as_elphapex(user_suffix=user_suffix))

    async def fault_light_on(self) -> bool:
        data = await self.web.blink(blink=True)
        if data:
            if data.get("code") == "B000":
                self.light = True
        return self.light

    async def fault_light_off(self) -> bool:
        data = await self.web.blink(blink=False)
        if data:
            if data.get("code") == "B100":
                self.light = False
        return self.light

    async def reboot(self) -> bool:
        data = await self.web.reboot()
        if data:
            return True
        return False

    async def _get_api_ver(self, web_summary: dict = None) -> Optional[str]:
        if web_summary is None:
            try:
                web_summary = await self.web.summary()
            except APIError:
                pass

        if web_summary is not None:
            try:
                self.api_ver = web_summary["STATUS"]["api_version"]
            except LookupError:
                pass

        return self.api_ver

    async def _get_fw_ver(self, web_get_system_info: dict = None) -> Optional[str]:
        if web_get_system_info is None:
            try:
                web_get_system_info = await self.web.get_system_info()
            except APIError:
                pass

        if web_get_system_info is not None:
            try:
                self.fw_ver = (
                    web_get_system_info["system_filesystem_version"]
                    .upper()
                    .split("V")[-1]
                )
            except LookupError:
                pass

        return self.fw_ver

    async def _get_hostname(self, web_get_system_info: dict = None) -> Optional[str]:
        if web_get_system_info is None:
            try:
                web_get_system_info = await self.web.get_system_info()
            except APIError:
                pass

        if web_get_system_info is not None:
            try:
                return web_get_system_info["hostname"]
            except KeyError:
                pass

    async def _get_mac(self, web_get_system_info: dict = None) -> Optional[str]:
        if web_get_system_info is None:
            try:
                web_get_system_info = await self.web.get_system_info()
            except APIError:
                pass

        if web_get_system_info is not None:
            try:
                return web_get_system_info["macaddr"]
            except KeyError:
                pass

        try:
            data = await self.web.get_network_info()
            if data:
                return data["macaddr"]
        except KeyError:
            pass

    async def _get_errors(self, web_summary: dict = None) -> List[MinerErrorData]:
        if web_summary is None:
            try:
                web_summary = await self.web.summary()
            except APIError:
                pass

        errors = []
        if web_summary is not None:
            try:
                for item in web_summary["SUMMARY"][0]["status"]:
                    try:
                        if not item["status"] == "s":
                            errors.append(X19Error(error_message=item["msg"]))
                    except KeyError:
                        continue
            except LookupError:
                pass
        return errors

    async def _get_hashboards(self, web_stats: dict | None = None) -> List[HashBoard]:
        hashboards = [
            HashBoard(slot=idx, expected_chips=self.expected_chips)
            for idx in range(self.expected_hashboards)
        ]

        if web_stats is None:
            try:
                web_stats = await self.web.stats()
            except APIError:
                return hashboards

        if web_stats is not None:
            try:
                for board in web_stats["STATS"][0]["chain"]:
                    hashboards[board["index"]].hashrate = self.algo.hashrate(
                        rate=board["rate_real"], unit=self.algo.unit.MH
                    ).into(self.algo.unit.default)
                    hashboards[board["index"]].chips = board["asic_num"]
                    board_temp_data = list(
                        filter(lambda x: not x == 0, board["temp_pcb"])
                    )
                    hashboards[board["index"]].temp = sum(board_temp_data) / len(
                        board_temp_data
                    )
                    chip_temp_data = list(
                        filter(lambda x: not x == "", board["temp_chip"])
                    )
                    hashboards[board["index"]].chip_temp = sum(
                        [int(i) / 1000 for i in chip_temp_data]
                    ) / len(chip_temp_data)
                    hashboards[board["index"]].serial_number = board["sn"]
                    hashboards[board["index"]].missing = False
            except LookupError:
                pass
        return hashboards

    async def _get_fault_light(
        self, web_get_blink_status: dict = None
    ) -> Optional[bool]:
        if self.light:
            return self.light

        if web_get_blink_status is None:
            try:
                web_get_blink_status = await self.web.get_blink_status()
            except APIError:
                pass

        if web_get_blink_status is not None:
            try:
                self.light = web_get_blink_status["blink"]
            except KeyError:
                pass
        return self.light

    async def _get_expected_hashrate(
        self, web_stats: dict = None
    ) -> Optional[AlgoHashRate]:
        if web_stats is None:
            try:
                web_stats = await self.web.stats()
            except APIError:
                pass

        if web_stats is not None:
            try:
                expected_rate = web_stats["STATS"][1]["total_rateideal"]
                try:
                    rate_unit = web_stats["STATS"][1]["rate_unit"]
                except KeyError:
                    rate_unit = "MH"
                return self.algo.hashrate(
                    rate=float(expected_rate), unit=self.algo.unit.from_str(rate_unit)
                ).into(self.algo.unit.default)
            except LookupError:
                pass

    async def _is_mining(self, web_get_miner_conf: dict = None) -> Optional[bool]:
        if web_get_miner_conf is None:
            try:
                web_get_miner_conf = await self.web.get_miner_conf()
            except APIError:
                pass

        if web_get_miner_conf is not None:
            try:
                if str(web_get_miner_conf["fc-work-mode"]).isdigit():
                    return (
                        False if int(web_get_miner_conf["fc-work-mode"]) == 1 else True
                    )
                return False
            except LookupError:
                pass

    async def _get_uptime(self, web_summary: dict = None) -> Optional[int]:
        if web_summary is None:
            try:
                web_summary = await self.web.summary()
            except APIError:
                pass

        if web_summary is not None:
            try:
                return int(web_summary["SUMMARY"][1]["elapsed"])
            except LookupError:
                pass

    async def _get_fans(self, web_stats: dict = None) -> List[Fan]:
        if web_stats is None:
            try:
                web_stats = await self.web.stats()
            except APIError:
                pass

        fans = [Fan() for _ in range(self.expected_fans)]
        if web_stats is not None:
            for fan_n in range(self.expected_fans):
                try:
                    fans[fan_n].speed = int(web_stats["STATS"][0]["fan"][fan_n])
                except LookupError:
                    pass

        return fans

    async def _get_pools(self, web_pools: list = None) -> List[PoolMetrics]:
        if web_pools is None:
            try:
                web_pools = await self.web.pools()
            except APIError:
                return []

        active_pool_index = None
        highest_priority = float("inf")

        for pool_info in web_pools["POOLS"]:
            if (
                pool_info.get("status") == "Alive"
                and pool_info.get("priority", float("inf")) < highest_priority
            ):
                highest_priority = pool_info["priority"]
                active_pool_index = pool_info["index"]

        pools_data = []
        if web_pools is not None:
            try:
                for pool_info in web_pools["POOLS"]:
                    url = pool_info.get("url")
                    pool_url = PoolUrl.from_str(url) if url else None
                    pool_data = PoolMetrics(
                        accepted=pool_info.get("accepted"),
                        rejected=pool_info.get("rejected"),
                        get_failures=pool_info.get("stale"),
                        remote_failures=pool_info.get("discarded"),
                        active=pool_info.get("index") == active_pool_index,
                        alive=pool_info.get("status") == "Alive",
                        url=pool_url,
                        user=pool_info.get("user"),
                        index=pool_info.get("index"),
                    )
                    pools_data.append(pool_data)
            except LookupError:
                pass
        return pools_data



from pyasic.miners.backends import M3X
from pyasic.miners.device.models import M31SPlusV10


class BTMinerM31SPlusV10(M3X, M31SPlusV10):
    pass


from pyasic.miners.device.models import M31SPlusV100


class BTMinerM31SPlusV100(M3X, M31SPlusV100):
    pass


from pyasic.miners.device.models import M31SPlusV20


class BTMinerM31SPlusV20(M3X, M31SPlusV20):
    pass


from pyasic.miners.device.models import M31SPlusV30


class BTMinerM31SPlusV30(M3X, M31SPlusV30):
    pass


from pyasic.miners.device.models import M31SPlusV40


class BTMinerM31SPlusV40(M3X, M31SPlusV40):
    pass


from pyasic.miners.device.models import M31SPlusV50


class BTMinerM31SPlusV50(M3X, M31SPlusV50):
    pass


from pyasic.miners.device.models import M31SPlusV60


class BTMinerM31SPlusV60(M3X, M31SPlusV60):
    pass


from pyasic.miners.device.models import M31SPlusV80


class BTMinerM31SPlusV80(M3X, M31SPlusV80):
    pass


from pyasic.miners.device.models import M31SPlusV90


class BTMinerM31SPlusV90(M3X, M31SPlusV90):
    pass


from pyasic.miners.device.models import M31SPlusVE10


class BTMinerM31SPlusVE10(M3X, M31SPlusVE10):
    pass


from pyasic.miners.device.models import M31SPlusVE20


class BTMinerM31SPlusVE20(M3X, M31SPlusVE20):
    pass


from pyasic.miners.device.models import M31SPlusVE30


class BTMinerM31SPlusVE30(M3X, M31SPlusVE30):
    pass


from pyasic.miners.device.models import M31SPlusVE40


class BTMinerM31SPlusVE40(M3X, M31SPlusVE40):
    pass


from pyasic.miners.device.models import M31SPlusVE50


class BTMinerM31SPlusVE50(M3X, M31SPlusVE50):
    pass


from pyasic.miners.device.models import M31SPlusVE60


class BTMinerM31SPlusVE60(M3X, M31SPlusVE60):
    pass


from pyasic.miners.device.models import M31SPlusVE80


class BTMinerM31SPlusVE80(M3X, M31SPlusVE80):
    pass


from pyasic.miners.device.models import M31SPlusVF20


class BTMinerM31SPlusVF20(M3X, M31SPlusVF20):
    pass


from pyasic.miners.device.models import M31SPlusVF30


class BTMinerM31SPlusVF30(M3X, M31SPlusVF30):
    pass


from pyasic.miners.device.models import M31SPlusVG20


class BTMinerM31SPlusVG20(M3X, M31SPlusVG20):
    pass


from pyasic.miners.device.models import M31SPlusVG30


class BTMinerM31SPlusVG30(M3X, M31SPlusVG30):
    pass



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import WhatsMinerMake


class M61VK10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M61VK10

    expected_chips = 180
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M61VK20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M61VK20

    expected_chips = 184
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M61VK30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M61VK30

    expected_chips = 188
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M61VK40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M61VK40

    expected_chips = 192
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M61VL10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M61VL10

    expected_chips = 135
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M61VL30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M61VL30

    expected_chips = 141
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M61VL40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M61VL40

    expected_chips = 144
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M61VL50(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M61VL50

    expected_chips = 147
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M61VL60(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M61VL60

    expected_chips = 150
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256



from __future__ import annotations

from typing_extensions import Self

from pyasic.device.algorithm.hashrate.base import AlgoHashRateType
from pyasic.device.algorithm.hashrate.unit.sha256 import SHA256Unit

from .unit import HashUnit


class SHA256HashRate(AlgoHashRateType):
    rate: float
    unit: SHA256Unit = HashUnit.SHA256.default

    def into(self, other: SHA256Unit) -> Self:
        return self.__class__(
            rate=self.rate / (other.value / self.unit.value), unit=other
        )



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import WhatsMinerMake


class M31HV10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M31HV10

    expected_chips = 114
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M31HV40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M31HV40

    expected_chips = 136
    expected_fans = 2
    expected_hashboards = 4
    algo = MinerAlgo.SHA256
